@@ -1,4 +1,4 @@
-from tastypie import bundle as tastypie_bundle, fields, utils
+from tastypie import bundle as tastypie_bundle, fields
 
 class ObjectId(fields.ApiField):
     """
@@ -29,15 +29,18 @@ class ApiNameMixin(object):
 class BuildRelatedMixin(ApiNameMixin):
     def build_related_resource(self, value, **kwargs):
         # A version of build_related_resource which allows only dictionary-alike data
-        if not hasattr(value, 'items'):
+        if hasattr(value, 'items'):
+            self.fk_resource = self.to_class(self.get_api_name())
+            # We force resource to cannot be updated so that
+            # it is just constructed by resource_from_data
+            self.fk_resource.can_update = lambda: False
+            return self.resource_from_data(self.fk_resource, value, **kwargs)
+        # Or if related object already exists (this happens with PATCH request)
+        elif getattr(value, 'obj', None):
+            return value
+        else:
             raise fields.ApiFieldError("The '%s' field was not given a dictionary-alike data: %s." % (self.instance_name, value))
 
-        self.fk_resource = self.to_class(self.get_api_name())
-        # We force resource to cannot be updated so that
-        # it is just constructed by resource_from_data
-        self.fk_resource.can_update = lambda: False
-        return self.resource_from_data(self.fk_resource, value, **kwargs)
-
 class ReferenceField(ApiNameMixin, fields.ToOneField):
     """
     References another MongoEngine document.
@@ -184,6 +187,9 @@ class EmbeddedListField(BuildRelatedMixin, fields.ToManyField):
             from tastypie_mongoengine import resources
             base = super(EmbeddedListField, self).to_class
             # We create a new ad-hoc resource class here, mixed with MongoEngineListResource, pretending to be original class
-            base_with_listresource = type(base.__name__, (base, resources.MongoEngineListResource), {'__module__': base.__module__})
-            self._to_class_with_listresource = lambda api_name=None: base_with_listresource(self._resource, self.instance_name, api_name)
+            self._to_class_with_listresource = type(base.__name__, (base, resources.MongoEngineListResource), {
+                '__module__': base.__module__,
+                '_parent': self._resource,
+                'attribute': self.instance_name,
+            })
         return self._to_class_with_listresource
