@@ -1,8 +1,9 @@
 import itertools, re, sys
 
-from django.conf.urls.defaults import url
+from django.conf import urls
 from django.core import exceptions
 from django.db.models import base
+from django.db.models.sql import constants
 from django.utils import datastructures
 
 from tastypie import bundle as tastypie_bundle, exceptions as tastypie_exceptions, fields as tastypie_fields, http, resources, utils
@@ -20,28 +21,77 @@ class NOT_HYDRATED:
     pass
 
 class ListQuerySet(datastructures.SortedDict):
+    def _process_filter_value(self, value):
+        # Sometimes value is passed as a list of one value
+        # (if filter was converted from QueryDict, for example)
+        if isinstance(value, (list, tuple)):
+            assert len(value) == 1
+            return value[0]
+        else:
+            return value
+
     def filter(self, **kwargs):
         result = self
 
         # pk optimization
         if 'pk' in kwargs:
-            pk = kwargs.pop('pk')
+            pk = self._process_filter_value(kwargs.pop('pk'))
             if pk in result:
-                result = ListQuerySet({pk: result[pk]})
-            else:
+                result = ListQuerySet([(pk, result[pk])])
+            # Sometimes None is passed as a pk to not filter by pk
+            elif pk is not None:
                 result = ListQuerySet()
 
         for field, value in kwargs.iteritems():
-            if '__' in field:
+            value = self._process_filter_value(value)
+            if constants.LOOKUP_SEP in field:
                 raise tastypie_exceptions.InvalidFilterError("Unsupported filter: (%s, %s)" % (field, value))
 
             try:
-                result = ListQuerySet({(obj.pk, obj) for obj in result.itervalues() if getattr(obj, field) == value})
+                result = ListQuerySet([(obj.pk, obj) for obj in result.itervalues() if getattr(obj, field) == value])
             except AttributeError, e:
-                raise queryset.InvalidQueryError('Cannot resolve field "%s"' % (field,))
+                raise tastypie_exceptions.InvalidFilterError(e)
 
         return result
 
+    def attrgetter(self, attr):
+        def g(obj):
+            return self.resolve_attr(obj, attr)
+        return g
+
+    def resolve_attr(self, obj, attr):
+        for name in attr.split(constants.LOOKUP_SEP):
+            while isinstance(obj, list):
+                # Try to be a bit similar to MongoDB
+                for o in obj:
+                    if hasattr(o, name):
+                        obj = o
+                        break
+                else:
+                    obj = obj[0]
+            obj = getattr(obj, name)
+        return obj
+
+    def order_by(self, *field_names):
+        if not len(field_names):
+            return self
+
+        result = self
+
+        for field in reversed(field_names):
+            if field.startswith('-'):
+                reverse = True
+                field = field[1:]
+            else:
+                reverse = False
+
+            try:
+                result = [(obj.pk, obj) for obj in sorted(result, key=self.attrgetter(field), reverse=reverse)]
+            except (AttributeError, IndexError), e:
+                raise tastypie_exceptions.InvalidSortError(e)
+
+        return ListQuerySet(result)
+
     def __iter__(self):
         return self.itervalues()
 
@@ -50,6 +100,9 @@ class ListQuerySet(datastructures.SortedDict):
         # a list here (order is same as our iteration order)
         if isinstance(key, (int, long)):
             return itertools.islice(self, key, key+1).next()
+        # Tastypie also access sliced object_list in paginator
+        elif isinstance(key, slice):
+            return itertools.islice(self, key.start, key.stop, key.step)
         else:
             return super(ListQuerySet, self).__getitem__(key)
 
@@ -132,10 +185,19 @@ class MongoEngineModelDeclarativeMetaclass(resources.ModelDeclarativeMetaclass):
         elif 'resource_type' in new_class.base_fields and not 'resource_type' in attrs:
             del(new_class.base_fields['resource_type'])
 
+        seen_types = set()
         for typ, resource in type_map.iteritems():
             if resource == 'self':
                 type_map[typ] = new_class
                 break
+            # In the code for polymorphic resources we are assuming
+            # that document classes are not duplicated among used resources
+            # (that each resource is linked to its own document class)
+            # So we are checking this assumption here
+            if type_map[typ]._meta.object_class in seen_types:
+                raise exceptions.ImproperlyConfigured("Used polymorphic resources should each use its own document class.")
+            else:
+                seen_types.add(type_map[typ]._meta.object_class)
 
         return new_class
 
@@ -159,12 +221,12 @@ class MongoEngineResource(resources.ModelResource):
 
         for name, obj in embedded:
             embedded_urls.extend((
-                url(r"^(?P<resource_name>%s)/(?P<pk>\w[\w-]*)/(?P<subresource_name>%s)%s$" % (self._meta.resource_name, name, utils.trailing_slash()),
+                urls.url(r"^(?P<resource_name>%s)/(?P<pk>\w[\w-]*)/(?P<subresource_name>%s)%s$" % (self._meta.resource_name, name, utils.trailing_slash()),
                     self.wrap_view('dispatch_subresource'),
                     {'request_type': 'list'},
                     name='api_dispatch_subresource_list',
                 ),
-                url(r"^(?P<resource_name>%s)/(?P<pk>\w[\w-]*)/(?P<subresource_name>%s)/(?P<index>\d+)%s$" % (self._meta.resource_name, name, utils.trailing_slash()),
+                urls.url(r"^(?P<resource_name>%s)/(?P<pk>\w[\w-]*)/(?P<subresource_name>%s)/(?P<index>\d+)%s$" % (self._meta.resource_name, name, utils.trailing_slash()),
                     self.wrap_view('dispatch_subresource'),
                     {'request_type': 'detail'},
                     name='api_dispatch_subresource_detail',
@@ -217,7 +279,12 @@ class MongoEngineResource(resources.ModelResource):
 
         object_type = self._get_object_type(request)
         if not object_type:
-            return fun()
+            # Polymorphic resources are enabled, but
+            # nothing is passed, so set it to a default
+            try:
+                object_type = self._get_type_from_class(type_map, self._meta.object_class)
+            except KeyError:
+                raise tastypie_exceptions.BadRequest("Invalid object type.")
 
         if object_type not in type_map:
             raise tastypie_exceptions.BadRequest("Invalid object type.")
@@ -317,7 +384,13 @@ class MongoEngineResource(resources.ModelResource):
                 # so we check for missing field here
                 # (https://github.com/toastdriven/django-tastypie/issues/496)
                 if not bundle.data.has_key(field_object.instance_name):
-                    value = None
+                    if field_object._default is not tastypie_fields.NOT_PROVIDED:
+                        if callable(field_object.default):
+                            value = field_object.default()
+                        else:
+                            value = field_object.default
+                    else:
+                        value = None
                 else:
                     value = field_object.hydrate(bundle)
                 if value is None:
@@ -465,7 +538,7 @@ class MongoEngineResource(resources.ModelResource):
         return result
 
     @classmethod
-    def get_fields(self, fields=None, excludes=None):
+    def get_fields(cls, fields=None, excludes=None):
         """
         Given any explicit fields to include and fields to exclude, add
         additional fields based on the associated document.
@@ -475,12 +548,12 @@ class MongoEngineResource(resources.ModelResource):
         fields = fields or []
         excludes = excludes or []
 
-        if not self._meta.object_class:
+        if not cls._meta.object_class:
             return final_fields
 
-        for name, f in self._meta.object_class._fields.iteritems():
+        for name, f in cls._meta.object_class._fields.iteritems():
             # If the field name is already present, skip
-            if name in self.base_fields:
+            if name in cls.base_fields:
                 continue
 
             # If field is not present in explicit field listing, skip
@@ -495,7 +568,7 @@ class MongoEngineResource(resources.ModelResource):
             #if cls.should_skip_field(f):
             #    continue
 
-            api_field_class = self.api_field_from_mongo_field(f)
+            api_field_class = cls.api_field_from_mongo_field(f)
 
             kwargs = {
                 'attribute': name,
@@ -536,13 +609,12 @@ class MongoEngineListResource(MongoEngineResource):
     A MongoEngine resource used in conjunction with EmbeddedListField.
     """
 
-    def __init__(self, parent, attribute, api_name=None):
-        self.parent = parent(api_name)
-        self.attribute = attribute
-        self.instance = None
-
+    def __init__(self, api_name=None):
         super(MongoEngineListResource, self).__init__(api_name)
 
+        self.instance = None
+        self.parent = self._parent(api_name)
+
     def _safe_get(self, request, **kwargs):
         filters = self.remove_api_resource_names(kwargs)
 
@@ -574,16 +646,6 @@ class MongoEngineListResource(MongoEngineResource):
 
         return kwargs_subset
 
-    def build_filters(self, filters=None):
-        # We do everything in apply_filters
-        return filters
-
-    def apply_sorting(self, obj_list, options=None):
-        if 'order_by' in options or 'sort_by' in options:
-            raise tastypie_exceptions.ImmediateHttpResponse(response=http.HttpNotImplemented())
-
-        return obj_list
-
     def get_object_list(self, request):
         if not self.instance:
             return ListQuerySet()
@@ -592,7 +654,7 @@ class MongoEngineListResource(MongoEngineResource):
             obj.pk = unicode(index)
             return obj
 
-        return ListQuerySet({(unicode(index), add_index(index, obj)) for index, obj in enumerate(getattr(self.instance, self.attribute))})
+        return ListQuerySet([(unicode(index), add_index(index, obj)) for index, obj in enumerate(getattr(self.instance, self.attribute))])
 
     def obj_create(self, bundle, request=None, **kwargs):
         bundle.obj = self._meta.object_class()
